function [faxis_vel,velocities,faxis_realpart_fine,realpart_fine,xr]=Find_Zero_Crossings(fmin,fmax,frequency,realpart,DIST,minv,maxv,p)
% This function calculates phase velocity using real part of cross-spectrum
% It is based on the algorithm explained in section 2.4 of Kastle et al., (2016)
% Kästle, Emanuel D., Riaz Soomro, Cornelis Weemstra, Lapo Boschi, and Thomas Meier. "Two-receiver measurements of phase velocity: cross-validation of ambient-noise and earthquake-based observations." Geophysical Journal International 207, no. 3 (2016): 1493-1512.

% Inputs:
% realpart         : Real part of the cross-spectrum which is the Fourier
%                    transform of the stacked cross-correlations between two stations, pay
%                    attention that only the positive frequencies are needed
% frequency        : Frequency vector corresponding to the vector of real part (Hz.), pay
%                    attention that only the positive frequencies are needed
% DIST             : station station distance, km
% fmin,fmax        : Valid range of frequency in the data, Hz
% minv, maxv       : Minimum and maximum velocity range to search in for phase velocities; km/s
% p                : Smoothing parameter for the spline interpolation function

% Outputs:
% velocities       : zero-crossings which are all possible phase velocities in the valid range of data, km/s
% faxis_vel        : Frequency axis corresponding to each zero-crossing, Hz
% faxis_realpart_fine       : Frequency axis corresponding to the real part of the cross-spectrum
% realpart_fine    : Real part of the cross-spectrum which is interpolated and smoothed on a very fine grid
% xr               : Zeros of the interpolated-smoothed real part of the cross-spectrum

% Copyright 2021 by Amin Rahimi Dalkhani (rahimi.petrol@gmail.com or a.rahimidalkhani@tudelft.nl), Kees Weemstra

% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

%% finding the zeros of zeroth order first kind bessel function
n=0;%zero order bessel function
kind=1;% first kind bessel function
k=100; %number of zeros
J0 = besselzero(n, k, kind);
accuracy=1e-10;% to be used in the root finding routine which is "findroots"

%% Interpolating real part of green function

indx_fmin=find(frequency>=fmin,1);
indx_fmax=find(frequency>fmax,1)-1;
frequency=frequency(indx_fmin:indx_fmax);
realpart=realpart(indx_fmin:indx_fmax);
%% Interpolating real part and then smoothing using Cubic smoothing spline (csaps)
% when the data contains noise we have to smooth the real part to reduce noise effects
pp=csaps(frequency,realpart,p);
df=0.00001;%frequency(2)-frequency(1);
faxis_realpart_fine=frequency(1):df:frequency(end);
realpart_fine=ppval(pp,faxis_realpart_fine);

%% Finding zeros of interpolated real part of Greens function
diffsign=diff(sign(realpart_fine));
i_roots=find(diffsign~=0);
nroots=length(i_roots);
xr=zeros(nroots,1);
for i=1:nroots
    x1=faxis_realpart_fine(i_roots(i));
    x2=faxis_realpart_fine(i_roots(i)+1);
    xr(i)=findroots(x1,x2,pp,accuracy);
end

slopes=diffsign(i_roots);
%% Finding velocities by equating zeros of bessel function to that of real part of green function (wn*distance/velocity=zeros of bessel function)

n=length(xr);
m=length(J0);
velocities=zeros(n*m,1);faxis_vel=velocities;
k=1;
for i=1:n
    if slopes(i)>0
        for j=2:2:m
            velocities(k)=2*pi*xr(i)*DIST/J0(j);
            faxis_vel(k)=xr(i);
            k=k+1;
        end
    elseif slopes(i)<0
        for j=1:2:m
            velocities(k)=2*pi*xr(i)*DIST/J0(j);
            faxis_vel(k)=xr(i);
            k=k+1;
        end
        
    end
end
i2r=find(velocities>maxv);
velocities(i2r)=[];faxis_vel(i2r)=[];
i2r=find(velocities<minv);
velocities(i2r)=[];faxis_vel(i2r)=[];
end
function x=findroots(x1,x2,pp,accuracy)
%% Finding a root of the function pp between [x1,x2] using the bisection root finding method
%% Written by Amin Rahimi Dalkhani, rahimi.petrol@gmail.com or a.rahimidalkhani@tudelft.nl
a=x1;fa=ppval(pp,a);
b=x2;fb=ppval(pp,b);
if sign(fa*fb)<0
    while b-a>accuracy
        x=(a+b)/2;
        fx=ppval(pp,x);
        if fx==0
            break
        elseif sign(fx)==sign(fa)
            a=x;fa=fx;
        else
            b=x;fb=fx;
        end
        
    end
else
    error('fa and fb should have different sign, please check the previous step')
end

end
