function PickedCurve=PhaseVelocityPicking(velocities,faxis,maxf,minf,DIST,freq2invert,nwavelength2discard,fdisplaylim,pvdisplaylimit)

% This function picks the phase velocity in an automatic way using a
% graphical interation with user so that the user can modify the picks and
% make sure to pick high quality data and discard uncertain points
%% Inputs:
% velocities                     : zero-crossings which are all possible phase velocities in the valid range of data,km/s
% faxis_vel                      : Frequency axis corresponding to each zero-crossing
% minf,maxf                      : Valid range of frequency in the data to pick (or picking limit), Hz
% pvdisplaylimit=[minv, maxv]    : Minimum and maximum velocity range to search in for phase velocities; km/s
% fdisplaylim                    : frequency axis display limit
% DIST                           : station-station distance, km
% freq2invert                    : Desired frequency components to be used in an inversion algorithm later
% nwavelength2discard            : Any phase velocity that leads to a wavelength that is smaller than DIST/nwavelength2discard will be ignored for inversion!
%% Output
% PickedCurve              : Picked phase velocity to be used in an inversion algorithm
% PickedCurve structure:
    % PickedCurve.DIST           : station-station distance, km
    % PickedCurve.discr_vels     : Picked phase velocity at the position of zeros of the real part
    % PickedCurve.discr_freqs    : The position of zeros of the real part
    % PickedCurve.freq2invert    : Frequency components to be used in the inversion requested by the user
    % PickedCurve.pv2invert      : Interpolated phase velocity at the location of the desired frequencies that are requested by the user
    % PickedCurve.TT2invert      : Computed frequency dependent travel times to be used in an inversion algorithm

% Copyright 2021 by Amin Rahimi Dalkhani (rahimi.petrol@gmail.com or a.rahimidalkhani@tudelft.nl), Kees Weemstra
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

% Reading reference curve
reference_curve=dlmread('disp_PREM300.data');
ref_freq_i=reference_curve(:,1);
ref_vel_i=1./reference_curve(:,2)/1000;%km/s
[fitresult_ref, ~] = createFit(ref_freq_i, ref_vel_i);
% Calculating minimum and maximum wavelength based on the reference curve
min_wavelength=fitresult_ref(maxf)/maxf;
% max_wavelength=fitresult_ref(minf)/minf;
% figure('Position',get(0,'screensize'))
% c1=1;PickedCurve{c1}.DIST=DIST;
skipcount=1;
skip=0;PickedCurve={};
if DIST>= min_wavelength
    %     velocities=velocities_t{i,j};
    %     faxis=faxis_vel_t{i,j};
    ref_freq=unique(faxis);
    %             ref_freq1=ref_freq;
    %             ref_freq1(ref_freq<minf)=[];
    ref_vel=fitresult_ref(ref_freq);
    ref_wavelength=ref_vel./ref_freq;
    [a,~]=hist(faxis,ref_freq);
    velocities1=zeros(max(a),length(ref_freq))+2*max(velocities(:));
    aa=cumsum(a);
    for ii=1:length(ref_freq)
        if ii==1
            i1=1;
        else
            i1=aa(ii-1)+1;
        end
        i2=aa(ii);
        velocities1(1:a(ii),ii)=velocities(i1:i2);
    end
    % ==== Finding the closest point to the reference velocity for the low frequency part
    discr_freqs=zeros(size(ref_freq));
    discr_vels=zeros(size(ref_freq));
%     k=find(ref_wavelength<=DIST,1);
    k4=find(ref_freq>=minf,4);% finding the first 4 points larger than the minimum picking frequency
    for j=1:4
        costfunc=abs(velocities1(1:a(k4(j)),k4(j))-ref_vel(k4(j)));
        [M,I]=min(costfunc);
        discr_freqs_k4(j)=ref_freq(k4(j));
        discr_vels_k4(j)=velocities1(I,k4(j));
        costfunc_k4(j)=M;
    end
%     [M,I]=min(costfunc);
%     k=k4(I);
%     k=k+1;
%     costfunc=abs(velocities1(1:a(k),k)-ref_vel(k));
%     [M,I]=min(costfunc);
    discr_freqs(k4)=discr_freqs_k4;
    discr_vels(k4)=discr_vels_k4;
    k=k4(end);
%     k=k+1;
%     costfunc=abs(velocities1(1:a(k),k)-ref_vel(k));
%     [M,I]=min(costfunc);
%     discr_freqs(k)=ref_freq(k);
%     discr_vels(k)=velocities1(I,k);
%     while abs(discr_vels(k)-ref_vel(k))<=abs(discr_vels(k-1)-ref_vel(k-1))
%         k=k+1;
%         costfunc=abs(velocities1(1:a(k),k)-ref_vel(k));
%         [M,I]=min(costfunc);
%         discr_freqs(k)=ref_freq(k);
%         discr_vels(k)=velocities1(I,k);
%     end
    % ==== Now pick the rest of the curve by finding the closest point to
    % the current point but for the next frequency which is a higher
    % frequency
    % TODO: this part can be improved by extrapolating the next point and
    % then find the closest point to the extrapolated one
    l=k+1;
    while ref_freq(l)<=maxf
        costfunc=abs(velocities1(1:a(l),l)-discr_vels(l-1));
        [M,I]=min(costfunc);
        discr_freqs(l)=ref_freq(l);
        discr_vels(l)=velocities1(I,l);
        l=l+1;
    end
    %% Removing picks smaller than minf, actually to remove zero values in the vector
    i2r=find(discr_freqs<minf);discr_freqs(i2r)=[];discr_vels(i2r)=[];
    %% Now editing the picked curve using ginput
    finished=0;
    while finished<1
        mes=sprintf(['p = pick;  d = delet every pick in the right side of selected point;  r = remove the selected point; a = pick again by considering the selected point as the starting point \n  s =  skip/ignore this pair because of poor quality; q = finished;  x = exit picking and save picked curves']);
%         mes=sprintf(['p = pick;  d = delet every pick in the right side of selected point;  r = remove the selected point; a = pick again by considering the selected point as the starting point \n  s =  skip/ignore this pair because of poor quality; q = finished;  x = exit picking and save picked curves;  left mouse button = zoom in;  right mouse button = zoom out \n',num2str(c1+skipcount),' of ',num2str(ncp),'\n','nsrc = ',num2str(i),' nrcv = ',num2str(j),', Interstation distance: ',num2str(DIST),' km']);
        fig=figure(gcf);hold off;
        plot(faxis,velocities,'bo','linewidth',2,'MarkerSize',10);
        %             title(num2str(DIST),'FontWeight','bold')
        hold on;plot(ref_freq,ref_vel,'y','linewidth',3)
        hold on;plot(discr_freqs,discr_vels,'k*','linewidth',3)
        ylim(pvdisplaylimit);xlim(fdisplaylim);
        if length(discr_freqs)>1
            % Adding interpolation result
            [fitresult_tt, gof] = createFit_fre_TT(discr_freqs,discr_vels);
            vv=fitresult_tt(freq2invert);
            hold on;plot(freq2invert,vv,'--r','MarkerSize',10,'Marker','+','linewidth',3)
            % Removing extrapolated points in the high frequency part
            i2=find(freq2invert(:)<=max(discr_freqs(:)));
            vv2=vv(i2);freq2invert2=freq2invert(i2);
            % Removing points that generates wavelength higher than the limit
            i3=find(nwavelength2discard*vv2(:)./freq2invert2(:)<=DIST);
            vv3=vv2(i3);freq2invert3=freq2invert2(i3);
            % Removing extrapolated points in the low frequency part
            i2invert=find(freq2invert3(:)>=min(discr_freqs(:)));
            hold on;plot(freq2invert3(i2invert),vv3(i2invert),'g','linestyle','non','MarkerSize',14,'Marker','o','linewidth',4)
        end
        
        legend('Zero crossings','Reference curve','Picked points','Interpolation result','Points to be used in inversion');
        %                 hold on;plot(freq,DIST./squeeze(stimes(i,j,1:nf)),'go')
%         title(mes,'Fontsize',14,'FontWeight','bold')
%         msgbox(mes)
%         set(gca,'FontSize',14,'FontWeight','bold');
        set(gca,'LineWidth',2,'FontWeight','bold','FontSize',16);
        xlabel('Frequency (Hz)','FontSize',14,'FontWeight','bold')
        ylabel('Phase velocity (km/s)','FontSize',14,'FontWeight','bold')
        xlim(fdisplaylim);
        ylim(pvdisplaylimit)
        [fpick,velpick,button]=ginput2(1);
        if button == 112%p Adding the point to the picks
            costfunc=abs(ref_freq-fpick);
            [M,I]=min(costfunc);
            fpick_disc=ref_freq(I);
            if any(discr_freqs==fpick_disc)
                
            else
                costfunc=abs(velocities1(1:a(I),I)-velpick);
                [M,II]=min(costfunc);
                velpick_disc=velocities1(II,I);
                discr_freqs=[discr_freqs;fpick_disc];
                discr_vels=[discr_vels;velpick_disc];
            end
        elseif button ==  114%r Removing the selected point from the data
            costfunc=abs(ref_freq-fpick);
            [M,I]=min(costfunc);
            fpick_disc=ref_freq(I);
            costfunc=abs(velocities1(1:a(I),I)-velpick);
            [M,II]=min(costfunc);
            discr_freq2rm=ref_freq(I);
            discr_vel2rm=velocities1(II,I);
            JJ=find(discr_freqs==discr_freq2rm);
            I2=find(discr_vels(JJ)==discr_vel2rm);
            I=JJ(I2);
            discr_freqs(I)=[];
            discr_vels(I)=[];
        elseif button == 100%d delet all picked points that are in the right side of the selected point
            costfunc=abs(discr_freqs-fpick);% Finding the closest frequency
            [M,I]=min(costfunc);
            i2d=find(discr_freqs>=discr_freqs(I));
            discr_freqs(i2d)=[];
            discr_vels(i2d)=[];
            % Here I can add repicking the rightsid of the selected point
        elseif button == 97%a pick again by considering the selected point as the starting point
            costfunc=abs(ref_freq-fpick);
            [M,I]=min(costfunc);
            fpick_disc=ref_freq(I);% Finding the closest frequency
            costfunc=abs(velocities1(1:a(I),I)-velpick);
            [M,II]=min(costfunc);
            k=I;
            discr_freqs=zeros(size(ref_freq));
            discr_vels=zeros(size(ref_freq));
            discr_freqs(k)=ref_freq(I);
            discr_vels(k)=velocities1(II,I);
            %                     discr_freqs(k)=ref_freq(k);
            %                     discr_vels(k)=velocities1(I,k);
            l=k+1;
            while ref_freq(l)<=maxf
                costfunc=abs(velocities1(1:a(l),l)-discr_vels(l-1));
                [M,I]=min(costfunc);
                discr_freqs(l)=ref_freq(l);
                discr_vels(l)=velocities1(I,l);
                l=l+1;
            end
            %% Removing picks smaller than minf, actually to remove zero values in the vector
            i2r=find(discr_freqs<eps);discr_freqs(i2r)=[];discr_vels(i2r)=[];
        elseif button == 120%x stop picking and save picked curves
            finished=1;
            error('Interrupted by user!');
        elseif button == 115%s Skip this pair, low quality data
            finished=1;
            skip=1;
            %                 elseif button == 98%b go back to previous zero crossings to pick again
            %                     finished=1;
            %                     back2previouspair=1;
            
        elseif button == 113%q Finished modification of picking
            finished=1;
        else% Nothing to do, repeat ginput
        end
    end
    
    if length(discr_freqs)>1
        % Adding interpolation result
        [fitresult_tt, gof] = createFit_fre_TT(discr_freqs,discr_vels);
        pv2invert=fitresult_tt(freq2invert);
         % Removing points that generates wavelength higher than the limit
%         i3=find(nwavelength2discard*pv2invert(:)./freq2invert(:)<=DIST);
%         pv2invert(i3)=0;
        pv2invert(nwavelength2discard*pv2invert(:)./freq2invert(:)>=DIST)=0;

        % Removing extrapolated points in the high frequency part by
        % asigning a zero phase velocity
%         i2=find(freq2invert(:)<=max(discr_freqs(:)));
%         pv2invert(i2)=0;
        pv2invert(freq2invert(:)>=max(discr_freqs(:)))=0;
        
               % Removing extrapolated points in the low frequency part
%         i4=find(freq2invert(:)<min(discr_freqs(:)));
%         pv2invert(i4)=0;
        pv2invert(freq2invert(:)<min(discr_freqs(:)))=0;
    end
    
    if button~=98 % b go back to previous zero crossings to pick again
        if skip==1
            skip=0;
            skipcount=skipcount+1;
        else
            PickedCurve.discr_vels=discr_vels;
            PickedCurve.discr_freqs=discr_freqs;
            PickedCurve.DIST=DIST;
            PickedCurve.pv2invert=pv2invert;% zero frequencies mean that frequency should be discarded from inversion
            PickedCurve.freq2invert=freq2invert;
            TT2invert=DIST./pv2invert;
            TT2invert(pv2invert<=0)=-1;
            PickedCurve.TT2invert=TT2invert;

        end
    end
    
end

end

function [fitresult, gof] = createFit_fre_TT(x, y)
%CREATEFIT(X,Y)
%  Create a fit.
%
%  Data for 'untitled fit 1' fit:
%      X Input : x
%      Y Output: y
%  Output:
%      fitresult : a fit object representing the fit.
%      gof : structure with goodness-of fit info.
%
%  See also FIT, CFIT, SFIT.

%  Auto-generated by MATLAB on 01-Nov-2020 22:50:18


%% Fit: 'untitled fit 1'.
[xData, yData] = prepareCurveData( x, y );

% Set up fittype and options.
% ft = 'pchipinterp'; % Shape-preserving piecewise cubic Hermite (pchip) interpolation
ft = 'cubicinterp'; % Cubic spline interpolation
% ft = 'linearinterp'; % Linear interpolation
[fitresult, gof] = fit( xData, yData, ft, 'Normalize', 'on' );
end

function [fitresult, gof] = createFit(ref_freq, ref_vel)
%CREATEFIT(REF_FREQ,REF_VEL)
%  Create a fit.
%
%  Data for 'untitled fit 1' fit:
%      X Input : ref_freq
%      Y Output: ref_vel
%  Output:
%      fitresult : a fit object representing the fit.
%      gof : structure with goodness-of fit info.
%
%  See also FIT, CFIT, SFIT.

%  Auto-generated by MATLAB on 02-Sep-2021 19:04:53


%% Fit: 'untitled fit 1'.
[xData, yData] = prepareCurveData( ref_freq, ref_vel );

% Set up fittype and options.
ft = 'linearinterp';

% Fit model to data.
[fitresult, gof] = fit( xData, yData, ft, 'Normalize', 'on' );
end